@@ -16,7 +16,7 @@ use Attribute::Handlers;
 # Version of this module.
 #
 
-our $VERSION = '1.21';
+our $VERSION = '1.30';
 
 ### PACKAGE PRIVATE VARIABLE ###
 #
@@ -47,15 +47,20 @@ my @POLL_HANDLERS = ();
 #
 
 sub UNIVERSAL::ExtDirect : ATTR(CODE) {
-    my ($package, $symbol, $referent, $attr, $data) = @_;
+    my ($package, $symbol, $referent, $attr, $data, $phase, $file, $line)
+        = @_;
 
-    croak "Attribute is not ExtDirect, what's wrong?"
+    croak "Method attribute is not ExtDirect at $file line $line"
         unless $attr eq 'ExtDirect';
 
-    croak "ExtDirect attribute must define method parameters"
+    croak "ExtDirect attribute must define ".
+          "method parameters at $file line $line"
         if !defined $data || ref $data ne 'ARRAY' || @$data < 0;
 
-    my $symbol_name = *{$symbol}{NAME};
+    my $symbol_name = eval { *{$symbol}{NAME} };
+    croak "Can't resolve symbol '$symbol' for package '$package' ".
+          "at $file line $line: $@"
+        if $@;
 
     # These parameters depend on attribute input
     my $param_no    = 0;
@@ -272,8 +277,6 @@ RPC::ExtDirect - Expose Perl code to JavaScript web applications through Ext.Dir
 
  package Foo::Bar;
     
- use Carp;
- 
  use RPC::ExtDirect Action => 'Fubar';
  
  sub foo : ExtDirect(2) {
@@ -285,17 +288,17 @@ RPC::ExtDirect - Expose Perl code to JavaScript web applications through Ext.Dir
     return $result;
  }
   
- sub bar : ExtDirect(params => [foo, bar]) {
+ sub bar : ExtDirect(params => ['foo', 'bar']) {
     my ($class, %arg) = @_;
   
     my $foo = $arg{foo};
     my $bar = $arg{bar};
   
     # do something, returning scalar
-    my $result = [ ... ];
+    my $result = eval { ... };
   
-    # or throw an exception if something's not right
-    croak "Houston, we've got a problem" if $error;
+    # or throw an exception if something's wrong
+    die "Houston, we've got a problem: $@\n" if $@;
   
     return $result;
  }
@@ -303,7 +306,7 @@ RPC::ExtDirect - Expose Perl code to JavaScript web applications through Ext.Dir
  sub baz : ExtDirect(formHandler) {
     my ($class, %arg) = @_;
   
-    my @form_fields    =  { grep !/^file_uploads$/ }, keys %arg;
+    my @form_fields    = grep { !/^file_uploads$/  } keys %arg;
     my @uploaded_files = @{ $arg{file_uploads}     };
   
     # do something with form fields and files
@@ -316,8 +319,8 @@ RPC::ExtDirect - Expose Perl code to JavaScript web applications through Ext.Dir
 
 =head2 Abstract
 
-This module provides easy way to map class methods to ExtDirect RPC interface
-used with ExtJS JavaScript framework.
+This module provides an easy way to map class methods to ExtDirect RPC
+interface used with ExtJS JavaScript framework.
 
 =head2 What is this for?
 
@@ -326,7 +329,7 @@ one called Ext.Direct. In short, Ext.Direct is a way to call server side code
 from client side without having to mess with HTML, forms and stuff like that.
 Besides forward asynchronous data stream (client calls server), Ext.Direct
 also provides mechanism for backward (server to client) asynchronous event
-generation.
+propagation.
 
 For more detailed explanation, see
 L<http://www.sencha.com/products/extjs/extdirect/>.
@@ -340,8 +343,8 @@ Ext.Direct uses the following terms, followed by their descriptions:
                   number and/or names.
  
  API            - JavaScript code that encodes Configuration.
-                  Usually generated on the fly by server side
-                  script called by client once upon startup.
+                  Usually generated by application server code
+                  called by client once upon startup.
  
  Router         - Server side component that receives remoting
                   calls, dispatches requests, collects and
@@ -352,28 +355,39 @@ Ext.Direct uses the following terms, followed by their descriptions:
                   languages may call it a Class. Since the
                   actual calling code is JavaScript, Action
                   names should conform to JavaScript naming
-                  rules (i.e. no ::, use dot instead).
+                  rules (i.e. no '::', use dots instead).
  
  Method         - Subroutine exposed through Ext.Direct API
                   to be called by client side. Method is
                   fully qualified by Action and Method names
                   using dot as delimiter: Action.Method.
-
- Result         - Any data returned by Method upon successful
-                  call completion.
  
- Exception      - An error, or any other unwanted condition
-                  on server side. Unlike Results, Exceptions
+ Result         - Any data returned by Method upon successful or
+                  unsuccessful call completion. This includes
+                  application logic errors. 'Not authenticated'
+                  and alike events should be returned as Results,
+                  not Exceptions.
+ 
+ Exception      - A fatal error, or any other unrecoverable event
+                  in application code. Unlike Results, Exceptions
                   are not considered successful; Ext.Direct
                   provides mechanism for managing Exceptions.
+                  Exceptions are not used to indicate errors in
+                  application logic flow, only for catastrophic
+                  conditions. Nearest analog is status code 500
+                  for HTTP responses.
   
  Event          - An asynchronous notification that can be
                   generated by server side and passed to
                   client side, resulting in some reaction.
+                  Events are useful for status updates, progress
+                  indicators and other predictably occuring
+                  conditions and events.
  
  Event Provider - Server side script that gets polled by
                   client side every N seconds; default N
-                  is 3 but it can be changed.
+                  is 3 but it can be changed in client side
+                  configuration.
 
 =head2 Using RPC::ExtDirect
 
@@ -384,7 +398,7 @@ declaration; it may be one of the following options:
     - Number of arguments to be passed as ordered list
     - Names of arguments to be passed as hash
     - formHandler that will receive hash of fields and uploaded files
-    - pollHandler does not receive any arguments
+    - pollHandler that does not receive any arguments
 
 Unlike Ext.Direct specification (and reference PHP implementation, too)
 RPC::ExtDirect does not impose any calling convention on server side code,
@@ -470,9 +484,8 @@ any given Action.
 
 =head1 DEPENDENCIES
 
-RPC::ExtDirect is dependent on the following modules:
-    Carp
-    Attribute::Handlers
+RPC::ExtDirect is dependent on the following modules: L<Attribute::Handlers>,
+L<JSON>.
 
 =head1 BUGS AND LIMITATIONS
 
@@ -499,7 +512,8 @@ Alexander Tokarev E<lt>tokarev@cpan.orgE<gt>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2011 by Alexander Tokarev.
+Copyright (c) 2011-2012 by Alexander Tokarev.
 
 This module is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself. See L<perlartistic>.
+
